==> nono-0.7.2.tar.gz: nono-0.7.2/vm/subram.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// サブメモリ
//

// SubRAM はバイト配置とする。

#include "subram.h"
#include "mainapp.h"

//
// 共通クラス
//

// コンストラクタ
SubRAMBaseDevice::SubRAMBaseDevice(int objid_, uint ramsize_kb)
	: inherited(objid_)
{
	ramsize = ramsize_kb * 1024;

	ram.reset(new uint8[ramsize]);
	memset(&ram[0], 0, ramsize);
}

// デストラクタ
SubRAMBaseDevice::~SubRAMBaseDevice()
{
}

// リセット
void
SubRAMBaseDevice::ResetHard(bool poweron)
{
	if (poweron) {
		// 実際にはゼロ初期化ではなくゴミだと思うがとりあえず。
		memset(&ram[0], 0, ramsize);
	}
}

// アドレスデコーダ
inline uint32
SubRAMBaseDevice::Decoder(uint32 addr) const
{
	// たぶん 128KB というかサイズ分で折り返しイメージが見えるはず。要確認。
	return addr % ramsize;
}

busdata
SubRAMBaseDevice::Read8(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data = ram[offset];
	putlog(4, "$%08x.B -> $%02x", addr, data);
	return data;
}

busdata
SubRAMBaseDevice::Read16(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data;
	data  = (uint32)ram[offset + 0] << 8;
	data |= (uint32)ram[offset + 1];
	putlog(4, "$%08x.W -> $%04x", addr, data);
	return data;
}

busdata
SubRAMBaseDevice::Read32(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data;
	data  = (uint32)ram[offset + 0] << 24;
	data |= (uint32)ram[offset + 1] << 16;
	data |= (uint32)ram[offset + 2] << 8;
	data |= (uint32)ram[offset + 3];
	putlog(4, "$%08x.L -> $%08x", addr, data);
	return data;
}

busdata
SubRAMBaseDevice::Write8(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.B <- $%02x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset] = data;
	return 0;
}

busdata
SubRAMBaseDevice::Write16(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.W <- $%04x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset + 0] = (data >> 8) & 0xff;
	ram[offset + 1] =  data       & 0xff;
	return 0;
}

busdata
SubRAMBaseDevice::Write32(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.L <- $%08x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset + 0] =  data >> 24;
	ram[offset + 1] = (data >> 16) & 0xff;
	ram[offset + 2] = (data >> 8)  & 0xff;
	ram[offset + 3] =  data        & 0xff;
	return 0;
}

busdata
SubRAMBaseDevice::Peek8(uint32 addr)
{
	uint32 offset = Decoder(addr);
	return ram[offset];
}

bool
SubRAMBaseDevice::Poke8(uint32 addr, uint32 data)
{
	if ((int32)data >= 0) {
		uint32 offset = Decoder(addr);
		ram[offset] = data;
	}
	return true;
}


//
// サブメモリ 1
//

// コンストラクタ
SubRAMDevice::SubRAMDevice(int ramsize_kb)
	: inherited(OBJ_SUBRAM, ramsize_kb)
{
}

// デストラクタ
SubRAMDevice::~SubRAMDevice()
{
}

// リセット
void
SubRAMDevice::ResetHard(bool poweron)
{
	// 通常モードなら電源オンで初期化。
	// MSXDOS モードなら MSXDOSDevice::Init() がすでにここに
	// 書き込んでいるため、消してはいけない。
	if (gMainApp.msxdos_mode == false) {
		inherited::ResetHard(poweron);
	}
}


//
// サブメモリ 2
//

// コンストラクタ
SubRAM2Device::SubRAM2Device(int ramsize_kb)
	: inherited(OBJ_SUBRAM2, ramsize_kb)
{
}

// デストラクタ
SubRAM2Device::~SubRAM2Device()
{
}

==> nono-0.7.3.tar.gz: nono-0.7.3/vm/subram.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// サブメモリ
//

// SubRAM はバイト配置とする。

#include "subram.h"
#include "mainapp.h"

//
// 共通クラス
//

// コンストラクタ
SubRAMBaseDevice::SubRAMBaseDevice(uint objid_, uint ramsize_kb)
	: inherited(objid_)
{
	ramsize = ramsize_kb * 1024;

	ram.reset(new uint8[ramsize]);
	memset(&ram[0], 0, ramsize);
}

// デストラクタ
SubRAMBaseDevice::~SubRAMBaseDevice()
{
}

// リセット
void
SubRAMBaseDevice::ResetHard(bool poweron)
{
	if (poweron) {
		// 実際にはゼロ初期化ではなくゴミだと思うがとりあえず。
		memset(&ram[0], 0, ramsize);
	}
}

// アドレスデコーダ
inline uint32
SubRAMBaseDevice::Decoder(uint32 addr) const
{
	// たぶん 128KB というかサイズ分で折り返しイメージが見えるはず。要確認。
	return addr % ramsize;
}

busdata
SubRAMBaseDevice::Read8(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data = ram[offset];
	putlog(4, "$%08x.B -> $%02x", addr, data);
	return data;
}

busdata
SubRAMBaseDevice::Read16(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data;
	data  = (uint32)ram[offset + 0] << 8;
	data |= (uint32)ram[offset + 1];
	putlog(4, "$%08x.W -> $%04x", addr, data);
	return data;
}

busdata
SubRAMBaseDevice::Read32(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data;
	data  = (uint32)ram[offset + 0] << 24;
	data |= (uint32)ram[offset + 1] << 16;
	data |= (uint32)ram[offset + 2] << 8;
	data |= (uint32)ram[offset + 3];
	putlog(4, "$%08x.L -> $%08x", addr, data);
	return data;
}

busdata
SubRAMBaseDevice::Write8(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.B <- $%02x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset] = data;
	return 0;
}

busdata
SubRAMBaseDevice::Write16(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.W <- $%04x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset + 0] = (data >> 8) & 0xff;
	ram[offset + 1] =  data       & 0xff;
	return 0;
}

busdata
SubRAMBaseDevice::Write32(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.L <- $%08x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset + 0] =  data >> 24;
	ram[offset + 1] = (data >> 16) & 0xff;
	ram[offset + 2] = (data >> 8)  & 0xff;
	ram[offset + 3] =  data        & 0xff;
	return 0;
}

busdata
SubRAMBaseDevice::Peek8(uint32 addr)
{
	uint32 offset = Decoder(addr);
	return ram[offset];
}

bool
SubRAMBaseDevice::Poke8(uint32 addr, uint32 data)
{
	if ((int32)data >= 0) {
		uint32 offset = Decoder(addr);
		ram[offset] = data;
	}
	return true;
}


//
// サブメモリ 1
//

// コンストラクタ
SubRAMDevice::SubRAMDevice(uint ramsize_kb)
	: inherited(OBJ_SUBRAM, ramsize_kb)
{
}

// デストラクタ
SubRAMDevice::~SubRAMDevice()
{
}

// リセット
void
SubRAMDevice::ResetHard(bool poweron)
{
	// 通常モードなら電源オンで初期化。
	// MSXDOS モードなら MSXDOSDevice::Init() がすでにここに
	// 書き込んでいるため、消してはいけない。
	if (gMainApp.msxdos_mode == false) {
		inherited::ResetHard(poweron);
	}
}


//
// サブメモリ 2
//

// コンストラクタ
SubRAM2Device::SubRAM2Device(uint ramsize_kb)
	: inherited(OBJ_SUBRAM2, ramsize_kb)
{
}

// デストラクタ
SubRAM2Device::~SubRAM2Device()
{
}
